@@ -1,4 +1,3 @@
-import threading
 import time
 import Queue
 import os
@@ -129,7 +128,6 @@ class Network(util.DaemonThread):
             config = {}  # Do not use mutables as default values!
         util.DaemonThread.__init__(self)
         self.config = SimpleConfig(config) if type(config) == type({}) else config
-        self.lock = threading.Lock()
         self.num_server = 8 if not self.config.get('oneserver') else 0
         self.blockchain = Blockchain(self.config, self)
         self.interfaces = {}
@@ -162,11 +160,19 @@ class Network(util.DaemonThread):
         if not os.path.exists(dir_path):
             os.mkdir(dir_path)
 
-        # address subscriptions and cached results
-        self.addresses = {}
+        # subscriptions and requests
+        self.subscribed_addresses = set()
+        # cached address status
+        self.addr_responses = {}
+        # unanswered requests
+        self.unanswered_requests = {}
+
         self.connection_status = 'connecting'
         self.requests_queue = Queue.Queue()
         self.set_proxy(deserialize_proxy(self.config.get('proxy')))
+        # retry times
+        self.server_retry_time = time.time()
+        self.nodes_retry_time = time.time()
 
     def read_recent_servers(self):
         if not self.config.path:
@@ -206,11 +212,16 @@ class Network(util.DaemonThread):
         self.notify('status')
 
     def is_connected(self):
-        return self.interface and self.interface.is_connected
+        return self.interface and self.interface.is_connected()
 
     def send_subscriptions(self):
-        for addr in self.addresses:
-            self.interface.send_request({'method':'blockchain.address.subscribe', 'params':[addr]})
+        # clear cache
+        self.cached_responses = {}
+        self.print_error('sending subscriptions to', self.interface.server, len(self.unanswered_requests), len(self.subscribed_addresses))
+        for r in self.unanswered_requests.values():
+            self.interface.send_request(r)
+        for addr in self.subscribed_addresses:
+            self.interface.send_request({'method':'blockchain.address.subscribe','params':[addr]})
         self.interface.send_request({'method':'server.banner','params':[]})
         self.interface.send_request({'method':'server.peers.subscribe','params':[]})
 
@@ -271,9 +282,9 @@ class Network(util.DaemonThread):
     def start_interface(self, server):
         if server in self.interfaces.keys():
             return
-        i = interface.Interface(server, self.config)
+        i = interface.Interface(server, self.queue, self.config)
         self.pending_servers.add(server)
-        i.start(self.queue)
+        i.start()
         return i
 
     def start_random_interface(self):
@@ -290,8 +301,6 @@ class Network(util.DaemonThread):
         self.running = True
         self.response_queue = response_queue
         self.start_interfaces()
-        t = threading.Thread(target=self.process_requests_thread)
-        t.start()
         self.blockchain.start()
         util.DaemonThread.start(self)
 
@@ -322,7 +331,7 @@ class Network(util.DaemonThread):
                 return
 
         if auto_connect:
-            if not self.interface.is_connected:
+            if not self.interface.is_connected():
                 self.switch_to_random_interface()
             else:
                 if self.server_is_lagging():
@@ -335,7 +344,7 @@ class Network(util.DaemonThread):
     def switch_to_random_interface(self):
         while self.interfaces:
             i = random.choice(self.interfaces.values())
-            if i.is_connected:
+            if i.is_connected():
                 self.switch_to_interface(i)
                 break
             else:
@@ -356,14 +365,14 @@ class Network(util.DaemonThread):
 
 
     def set_server(self, server):
-        if self.default_server == server and self.interface.is_connected:
+        if self.default_server == server and self.interface.is_connected():
             return
 
         if self.protocol != deserialize_server(server)[2]:
             return
 
         # stop the interface in order to terminate subscriptions
-        if self.interface.is_connected:
+        if self.interface.is_connected():
             self.stop_interface()
 
         # notify gui
@@ -404,6 +413,10 @@ class Network(util.DaemonThread):
 
 
     def process_response(self, i, response):
+        # the id comes from the daemon or the network proxy
+        _id = response.get('id')
+        if _id is not None:
+            self.unanswered_requests.pop(_id)
         method = response['method']
         if method == 'blockchain.address.subscribe':
             self.on_address(i, response)
@@ -416,12 +429,12 @@ class Network(util.DaemonThread):
         else:
             self.response_queue.put(response)
 
-    def process_requests_thread(self):
-        while self.is_running():
+    def handle_requests(self):
+        while True:
             try:
-                request = self.requests_queue.get(timeout=0.1)
+                request = self.requests_queue.get_nowait()
             except Queue.Empty:
-                continue
+                break
             self.process_request(request)
 
     def process_request(self, request):
@@ -441,55 +454,54 @@ class Network(util.DaemonThread):
                 out['error'] = str(e)
                 traceback.print_exc(file=sys.stdout)
                 self.print_error("network error", str(e))
-
             self.response_queue.put(out)
             return
 
         if method == 'blockchain.address.subscribe':
             addr = params[0]
-            if addr in self.addresses:
-                self.response_queue.put({'id':_id, 'result':self.addresses[addr]})
+            self.subscribed_addresses.add(addr)
+            if addr in self.addr_responses:
+                self.response_queue.put({'id':_id, 'result':self.addr_responses[addr]})
                 return
 
-        try:
-            self.interface.send_request(request)
-        except:
-            # put it back in the queue
-            self.print_error("warning: interface not ready for", request)
-            self.requests_queue.put(request)
-            time.sleep(0.1)
+        # store unanswered request
+        self.unanswered_requests[_id] = request
+        self.interface.send_request(request)
+
+    def check_interfaces(self):
+        now = time.time()
+        # nodes
+        if len(self.interfaces) + len(self.pending_servers) < self.num_server:
+            self.start_random_interface()
+            if now - self.nodes_retry_time > NODES_RETRY_INTERVAL:
+                self.print_error('network: retrying connections')
+                self.disconnected_servers = set([])
+                self.nodes_retry_time = now
+        # main interface
+        if not self.interface.is_connected():
+            if self.config.get('auto_cycle'):
+                if self.interfaces:
+                    self.switch_to_random_interface()
+            else:
+                if self.default_server in self.interfaces.keys():
+                    self.switch_to_interface(self.interfaces[self.default_server])
+                else:
+                    if self.default_server in self.disconnected_servers:
+                        if now - self.server_retry_time > SERVER_RETRY_INTERVAL:
+                            self.disconnected_servers.remove(self.default_server)
+                            self.server_retry_time = now
+                    else:
+                        if self.default_server not in self.pending_servers:
+                            self.print_error("forcing reconnection")
+                            self.interface = self.start_interface(self.default_server)
 
     def run(self):
-        server_retry_time = time.time()
-        nodes_retry_time = time.time()
         while self.is_running():
+            self.check_interfaces()
+            self.handle_requests()
             try:
                 i, response = self.queue.get(timeout=0.1)
             except Queue.Empty:
-                now = time.time()
-                if len(self.interfaces) + len(self.pending_servers) < self.num_server:
-                    self.start_random_interface()
-                if not self.interfaces:
-                    if now - nodes_retry_time > NODES_RETRY_INTERVAL:
-                        self.print_error('network: retrying connections')
-                        self.disconnected_servers = set([])
-                        nodes_retry_time = now
-                if not self.interface.is_connected:
-                    if self.config.get('auto_cycle'):
-                        if self.interfaces:
-                            self.switch_to_random_interface()
-                    else:
-                        if self.default_server in self.interfaces.keys():
-                            self.switch_to_interface(self.interfaces[self.default_server])
-                        else:
-                            if self.default_server in self.disconnected_servers:
-                                if now - server_retry_time > SERVER_RETRY_INTERVAL:
-                                    self.disconnected_servers.remove(self.default_server)
-                                    server_retry_time = now
-                            else:
-                                if self.default_server not in self.pending_servers:
-                                    self.print_error("forcing reconnection")
-                                    self.interface = self.start_interface(self.default_server)
                 continue
 
             if response is not None:
@@ -500,12 +512,11 @@ class Network(util.DaemonThread):
             if i.server in self.pending_servers:
                 self.pending_servers.remove(i.server)
 
-            if i.is_connected:
+            if i.is_connected():
                 self.add_interface(i)
                 self.add_recent_server(i)
                 i.send_request({'method':'blockchain.headers.subscribe','params':[]})
                 if i == self.interface:
-                    self.print_error('sending subscriptions to', self.interface.server)
                     self.send_subscriptions()
                     self.set_status('connected')
             else:
@@ -555,7 +566,7 @@ class Network(util.DaemonThread):
     def on_address(self, i, r):
         addr = r.get('params')[0]
         result = r.get('result')
-        self.addresses[addr] = result
+        self.addr_responses[addr] = result
         self.response_queue.put(r)
 
     def get_header(self, tx_height):
