@@ -71,14 +71,12 @@ class SimpleConfig(object):
         # update the current options with the command line options last (to
         # override both others).
         self.read_only_options.update(options)
-
         # init path
         self.init_path()
-
         # user config.
         self.user_config = read_user_config_function(self.path)
-
-        set_config(self)  # Make a singleton instance of 'self'
+        # Make a singleton instance of 'self'
+        set_config(self)
 
     def init_path(self):
         # Read electrum path in the command line configuration
@@ -104,7 +102,6 @@ class SimpleConfig(object):
             self.user_config[key] = value
             if save:
                 self.save_user_config()
-
         return
 
     def get(self, key, default=None):
@@ -134,6 +131,35 @@ class SimpleConfig(object):
             import stat
             os.chmod(path, stat.S_IREAD | stat.S_IWRITE)
 
+    def get_wallet_path(self):
+        """Set the path of the wallet."""
+
+        # command line -w option
+        path = self.get('wallet_path')
+        if path:
+            return path
+
+        # path in config file
+        path = self.get('default_wallet_path')
+        if path and os.path.exists(path):
+            return path
+
+        # default path
+        dirpath = os.path.join(self.path, "wallets")
+        if not os.path.exists(dirpath):
+            os.mkdir(dirpath)
+
+        new_path = os.path.join(self.path, "wallets", "default_wallet")
+
+        # default path in pre 1.9 versions
+        old_path = os.path.join(self.path, "electrum.dat")
+        if os.path.exists(old_path) and not os.path.exists(new_path):
+            os.rename(old_path, new_path)
+
+        return new_path
+
+
+
 def read_system_config(path=SYSTEM_CONFIG_PATH):
     """Parse and return the system config settings in /etc/electrum.conf."""
     result = {}
